/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.multiplexer;

import java.io.Serializable;
import java.util.Map;

import org.objectweb.dream.Pull;
import org.objectweb.dream.Push;

/**
 * This class defines an Attachement.
 * 
 * @author <a href="mailto:Vivien.Quema@inrialpes.fr">Vivien Quema </a>
 * @version 1.0
 */
public class Attachment implements Serializable, Cloneable
{

  /** The id of the attachment. */
  protected int      id;

  /** The name of attached inputs. */
  protected String[] inputNames;

  /** The attached inputs. */
  protected Pull[]   inputs;

  /** The context to be passed to inputs when they are used. */
  protected Map[]    inputContexts;

  /** The name of the outputs to which inputs are attached. */
  protected String[] outputNames;

  /** The outputs to which inputs are attached. */
  protected Push[]   outputs;

  /** The context to be passed to outputs when they are used. */
  protected Map[]    outputContexts;

  /**
   * Creates a new <code>Attachment</code> object
   * 
   * @param id the id of the attachment.
   * @param inputNames the names of the attached inputs.
   * @param inputs the attached inputs.
   * @param inputContexts the context associated with attached inputs.
   * @param outputNames the names of the attached outputs.
   * @param outputs the attached outputs.
   * @param outputContexts the context associated with attached outputs.
   */
  public Attachment(int id, String[] inputNames, Pull[] inputs,
      Map[] inputContexts, String[] outputNames, Push[] outputs,
      Map[] outputContexts)
  {
    this.id = id;
    this.inputNames = inputNames;
    this.inputs = inputs;
    this.inputContexts = inputContexts;
    this.outputNames = outputNames;
    this.outputs = outputs;
    this.outputContexts = outputContexts;
  }

  /**
   * @return the id.
   */
  public int getId()
  {
    return id;
  }

  /**
   * @return the inputContexts.
   */
  public Map[] getInputContexts()
  {
    return inputContexts;
  }

  /**
   * @param inputContexts the inputContexts to set.
   */
  public void setInputContexts(Map[] inputContexts)
  {
    this.inputContexts = inputContexts;
  }

  /**
   * @return the inputNames.
   */
  public String[] getInputNames()
  {
    return inputNames;
  }

  /**
   * @param inputNames the inputNames to set.
   */
  public void setInputNames(String[] inputNames)
  {
    this.inputNames = inputNames;
  }

  /**
   * @return the outputContexts.
   */
  public Map[] getOutputContexts()
  {
    return outputContexts;
  }

  /**
   * @param outputContexts the outputContexts to set.
   */
  public void setOutputContexts(Map[] outputContexts)
  {
    this.outputContexts = outputContexts;
  }

  /**
   * @return the outputNames.
   */
  public String[] getOutputNames()
  {
    return outputNames;
  }

  /**
   * @param outputNames the outputNames to set.
   */
  public void setOutputNames(String[] outputNames)
  {
    this.outputNames = outputNames;
  }

  /**
   * @return the inputs.
   */
  public Pull[] getInputs()
  {
    return inputs;
  }

  /**
   * @param inputs the inputs to set.
   */
  public void setInputs(Pull[] inputs)
  {
    this.inputs = inputs;
  }

  /**
   * @return the outputs.
   */
  public Push[] getOutputs()
  {
    return outputs;
  }

  /**
   * @param outputs the outputs to set.
   */
  public void setOutputs(Push[] outputs)
  {
    this.outputs = outputs;
  }

  /**
   * @see java.lang.Object#hashCode()
   */
  public int hashCode()
  {
    return id;
  }

  /**
   * Returns <code>true</code> iff the specified object is an Attachment with
   * the same id.
   * 
   * @see java.lang.Object#equals(java.lang.Object)
   */
  public boolean equals(Object obj)
  {
    if (obj instanceof Attachment)
    {
      return (id == ((Attachment) obj).id);
    }
    return false;
  }

  /**
   * Returns a clone of this attachment.
   * <p>
   * <i>Notes: </i> inputs, inputContexts, outputs, and outputs are set to
   * <code>null</code>.
   * 
   * @return a clone of this attachment.
   */
  public Object clone()
  {
    try
    {
      Attachment obj = (Attachment) super.clone();
      // the inputs and outputs
      obj.inputs = null;
      obj.outputs = null;
      obj.inputContexts = null;
      obj.outputContexts = null;
      // the inputNames
      obj.inputNames = new String[inputNames.length];
      for (int i = 0; i < inputNames.length; i++)
      {
        obj.inputNames[i] = inputNames[i];
      }
      // the outputNames
      obj.outputNames = new String[outputNames.length];
      for (int i = 0; i < outputNames.length; i++)
      {
        obj.outputNames[i] = outputNames[i];
      }
      return obj;
    }
    catch (CloneNotSupportedException e)
    {
      // this shouldn't happen, since we are Cloneable
      throw new InternalError();
    }
  }
}